@@ -575,7 +575,7 @@ end_gnutls_io (GTlsConnectionGnutls  *gnutls,
             ret == GNUTLS_E_WARNING_ALERT_RECEIVED) &&	\
            !gnutls->priv->error);			\
   ret = end_gnutls_io (gnutls, ret, error);		\
-  if (ret < 0 && error && !*error)			\
+  if (ret < 0 && ret != GNUTLS_E_REHANDSHAKE && error && !*error) \
     {							\
       g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_MISC,\
                    errmsg, gnutls_strerror (ret));	\
@@ -843,7 +843,8 @@ handshake_internal (GTlsConnectionGnutls  *gnutls,
   int ret;
 
   if (G_IS_TLS_SERVER_CONNECTION_GNUTLS (gnutls) &&
-      gnutls->priv->ever_handshaked && !gnutls->priv->need_handshake)
+      gnutls->priv->ever_handshaked && !gnutls->priv->handshaking &&
+      !gnutls->priv->need_handshake)
     {
       BEGIN_GNUTLS_IO (gnutls, blocking, cancellable);
       ret = gnutls_rehandshake (gnutls->priv->session);
